BlandAltman<-function(x="",y=c("",""),labx="Mean",laby="Difference",maintit="Bland-Altman difference plot")
# Version 0.1	06.11.2004
{
if (length(x) != length(y))
	{
	cat(rep("\n",20))
	cat("The function \n\n\t\tBlandAltman (x, y, xlabel, ylabel, maintitle) \n\n* displays the classic Bland-Altman difference plot with limits ")
	cat("of agreement, \n* prints the mean difference and both agreement limits (along with 95% CIs) and \n* returns a list of relevant ")
	cat("parameters (the mean difference, its SD and SE, \n  upper and lower agreement limits, their SE, and the t value ")
	cat("used in the \n  calculation of CIs).\n\n")
	cat("The first two arguments (x and y) are required and should be vectors of equal \nlength, otherwise the execution is halted. ")
	cat("The other arguments are optional. \nWhen called with no arguments, this help is displayed.\n\n")
	cat("Based on the original Bland & Altman paper (Lancet 1986)\n(http://www-users.york.ac.uk/~mb55/meas/ba.htm)\n")
	cat("Author: Jaro Lajovic <jaro.lajovic@mf.uni-lj.si>\n\n\n\n")
 	}
else
	{
	difference<-x-y								# vector of differences
	average<-(x+y)/2							# vector of means
	difference.mean<-mean(difference)			# mean difference
	difference.sd<-sd(difference)				# SD of differences
	al<-2*difference.sd
	upper.agreement.limit<-difference.mean+al	# agreement limits
	lower.agreement.limit<-difference.mean-al
	n<-length(difference)						# number of 'observations'
	
	difference.se<-difference.sd/sqrt(n)		# standard error of the mean
	al.se<-difference.sd*sqrt(3)/sqrt(n)		# standard error of the agreement limit
	tvalue<-qt(0.975,n-1)						# t value for 95% CI calculation
	difference.mean.95ci<-difference.se*tvalue
	al.95ci<-al.se*tvalue
	upper.agreement.limit.95ci<-c(upper.agreement.limit-al.95ci,upper.agreement.limit+al.95ci)
	lower.agreement.limit.95ci<-c(lower.agreement.limit-al.95ci,lower.agreement.limit+al.95ci)
	
	# The x and the y limits of the plot (limx, limy)
	if(min(average)<0){x.lower.boundary<-min(average)} else {x.lower.boundary<-0}
	x.upper.boundary<-max(average)
	limx<-c(x.lower.boundary,x.upper.boundary)
	y.boundary<-max(abs(pretty(difference)))
	limy<-c(-y.boundary,y.boundary)
	
	# Plot
	plot.default(average,difference,cex=0.7,xlim=limx,ylim=limy,xlab=labx,ylab=laby,main=maintit)
	abline(h=upper.agreement.limit,lty="dotted",col="blue")
	abline(h=difference.mean,col="blue")
	abline(h=lower.agreement.limit,lty="dotted",col="blue")

	# Print results
	cat(rep("\n",30))
	cat("Mean difference = ",difference.mean,"   (95% CI: ",difference.mean-difference.mean.95ci," - ",difference.mean+difference.mean.95ci,")\n")
	cat("Upper agreement limit = ",upper.agreement.limit,"   (95% CI: ",upper.agreement.limit.95ci[1]," - ",upper.agreement.limit.95ci[2],")\n")
	cat("Lower agreement limit = ",lower.agreement.limit,"   (95% CI: ",lower.agreement.limit.95ci[1]," - ",lower.agreement.limit.95ci[2],")\n")
	cat("\n\n")
	
	# Return list
	result<-list(difference.mean=difference.mean,difference.sd=difference.sd,difference.se=difference.se,upper.agreement.limit=upper.agreement.limit,lower.agreement.limit=lower.agreement.limit,agreement.limit.se=al.se,t.value=tvalue)
	}
}
